/*  Using LVGL with Arduino requires some extra steps:
    Be sure to read the docs here: https://docs.lvgl.io/master/get-started/platforms/arduino.html  */

#include <lvgl.h>
#include <Arduino_GFX_Library.h>

/*Change to your screen resolution*/
static const uint16_t screenWidth  = 480;
static const uint16_t screenHeight = 272;

static lv_disp_draw_buf_t draw_buf;
static lv_color_t buf[ screenWidth * 10 ];

#define GFX_BL 2        // GPIO pin connected to the display backlight
#define TFT_BL GFX_BL

Arduino_ESP32RGBPanel* rgbpanel = new Arduino_ESP32RGBPanel(
    40 /* DE */, 41 /* VSYNC */, 39 /* HSYNC */, 42 /* PCLK */,
    45 /* R0 */, 48 /* R1 */, 47 /* R2 */, 21 /* R3 */, 14 /* R4 */,
    5 /* G0 */, 6 /* G1 */, 7 /* G2 */, 15 /* G3 */, 16 /* G4 */, 4 /* G5 */,
    8 /* B0 */, 3 /* B1 */, 46 /* B2 */, 9 /* B3 */, 1 /* B4 */,
    0 /* hsync_polarity */, 1 /* hsync_front_porch */, 1 /* hsync_pulse_width */, 43 /* hsync_back_porch */,
    0 /* vsync_polarity */, 3 /* vsync_front_porch */, 1 /* vsync_pulse_width */, 12 /* vsync_back_porch */,
    1 /* pclk_active_neg */, 9000000 /* prefer_speed */ );

Arduino_RGB_Display* gfx = new Arduino_RGB_Display( 480 /* width */, 272 /* height */, rgbpanel );

#include "touch.h"
#include "demos/lv_demos.h"

#if LV_USE_LOG != 0
    /* Serial debugging */
    void my_print( const char* buf ) {
        Serial.printf( buf );
        Serial.flush();
    }
#endif

/* Display flushing */
void my_disp_flush( lv_disp_drv_t* disp, const lv_area_t* area, lv_color_t* color_p ) {
    uint32_t w = ( area->x2 - area->x1 + 1 );
    uint32_t h = ( area->y2 - area->y1 + 1 );

    #if ( LV_COLOR_16_SWAP == 0 )
        gfx->draw16bitRGBBitmap( area->x1, area->y1, ( uint16_t* )&color_p->full, w, h );
    #else
        gfx->draw16bitBeRGBBitmap( area->x1, area->y1, ( uint16_t* )&color_p->full, w, h );
    #endif

    lv_disp_flush_ready( disp );
}

/*Read the touchpad*/
void my_touchpad_read( lv_indev_drv_t* indev_driver, lv_indev_data_t* data ) {

    if ( touch_has_signal() ) {
        if ( touch_touched() ) {
            data->state = LV_INDEV_STATE_PR;

            /*Set the coordinates*/
            data->point.x = touch_last_x;
            data->point.y = touch_last_y;
        }
        else if ( touch_released() )
            data->state = LV_INDEV_STATE_REL;
    }
    else
        data->state = LV_INDEV_STATE_REL;
}


void setup() {

    gfx->begin();
    gfx->fillScreen( BLACK );

    pinMode( TFT_BL, OUTPUT );
    digitalWrite( TFT_BL, HIGH );

    Serial.begin( 115200 ); /* prepare for possible serial debug */
    delay( 250 );

    String LVGL_Arduino = "Hello Arduino! ";
    LVGL_Arduino += String( 'V' ) + lv_version_major() + "." + lv_version_minor() + "." + lv_version_patch();

    Serial.println( "\r\n" );
    Serial.println( LVGL_Arduino );
    Serial.println( "I am LVGL_Arduino" );

    lv_init();

    #if LV_USE_LOG != 0
        lv_log_register_print_cb( my_print ); /* register print function for debugging */
    #endif

    lv_disp_draw_buf_init( &draw_buf, buf, NULL, screenWidth * 10 );

    /* Initialize the display */
    static lv_disp_drv_t disp_drv;
    lv_disp_drv_init( &disp_drv );

    /* Change the following line to your display resolution */
    disp_drv.hor_res = screenWidth;
    disp_drv.ver_res = screenHeight;

    disp_drv.flush_cb = my_disp_flush;
    disp_drv.draw_buf = &draw_buf;
    lv_disp_drv_register( &disp_drv );

    /* Initialize the (dummy) input device driver */
    static lv_indev_drv_t indev_drv;
    lv_indev_drv_init( &indev_drv );
    indev_drv.type = LV_INDEV_TYPE_POINTER;
    indev_drv.read_cb = my_touchpad_read;
    lv_indev_drv_register( &indev_drv );

    #if 0
        /* Create simple label */
        lv_obj_t* label = lv_label_create( lv_scr_act() );
        lv_label_set_text( label, LVGL_Arduino.c_str() );
        lv_obj_align( label, LV_ALIGN_CENTER, 0, 0 );
    #else
        /*  Try an example from the lv_examples Arduino library
            make sure to include it as written above.
            lv_example_btn_1();
        */

        /*
            To use the built-in examples and demos of LVGL, uncomment the includes below respectively.
            - You also need to copy `lvgl/examples` to `lvgl/src/examples`. Similarly for the demos, `lvgl/demos` to `lvgl/src/demos`.
            - Note that the `lv_examples` library is for LVGL v7 and you shouldn't install it for this version (since LVGL v8)
            as the examples and demos are now part of the main LVGL library.
        */

        // uncomment one of these demos
        lv_demo_widgets();            // OK
        // lv_demo_benchmark();          // OK
        // lv_demo_keypad_encoder();     // works, but I haven't an encoder
        // lv_demo_music();              // NOK
        // lv_demo_printer();
        // lv_demo_stress();             // seems to be OK
    #endif

    Serial.println( "Setup done" );
}

void loop() {
    lv_timer_handler(); /* let the GUI do its work */
    delay( 5 );
}






//  --- EOF ---
